import os

from django.shortcuts import render, render_to_response, HttpResponse
from django.conf import settings

import suggestions
import main
from main.models import Experiment, GeneInfo, Species

import numpy
import re
import json


# Create your views here.

class GeneInfoCorrelation:
    transcript_name = ""
    description = ""
    gene_name = ""
    chr = 0
    start = 0
    correlation = 0


def correlation(request):
    '''
    select a gene and experiment, and find the top genes that show correlation
    
    '''
    species_name = "Arabidopsis thaliana"
    experiments = Experiment.objects.filter(species__species_name=species_name)
    species_short_name = Species.objects.get(species_name=species_name).short_name

    if request.method == 'GET':
        if request.GET.get('query'):
            if request.GET.get('experiment_name'):
                exp_name = request.GET.get('experiment_name')
            else:
                exp_name = experiments[0].experiment_name
            gene_id = request.GET.get('query').strip()

            if not gene_id:
                return render_to_response('correlation.html', {'experiments': experiments})

            geneInfoList = GeneInfo.objects.filter(gene_id = gene_id, species__species_name = species_name)
            if geneInfoList.count() == 0:
                geneInfoList = GeneInfo.objects.filter(gene_name__iexact=gene_id)

                if geneInfoList.count() == 0:
                    geneInfoList = GeneInfo.objects.filter(description__icontains=gene_id)

                if geneInfoList.count() != 1:
                    ret = suggestions.views.suggest(gene_id, exp_name, "correlation")
                    if ret:
                        return ret
                    else:
                        return main.views.no_results(gene_id, "correlation")

            geneInfo = geneInfoList[0]

            queryGene = dict()
            queryGene["id"] = geneInfo.gene_id
            queryGene["name"] = geneInfo.gene_name
            queryGene["chromosome"] = geneInfo.chr
            queryGene["start"] = geneInfo.start

            corrthld = 0.9
            if request.GET.get('corrthld'):
                try:
                    corrthld = float(request.GET.get('corrthld'))
                    if corrthld > 1:
                        corrthld = 1
                    elif corrthld < -1:
                        corrthld = -1
                except ValueError:
                    return HttpResponse('<h1> invalid correlation threshold </h1>')

                # lodscores = loadLodFile(exp_name)
                # correlations = calculateCorrelation(gene_name.upper(),lodscores)
            correlations = loadPearsonFile(queryGene["id"].upper(), exp_name)
            genes = getCorrelatingGenes(correlations, corrthld)

            genelist = list()
            for gene in genes:
                g = GeneInfoCorrelation()
                g.gene_id = gene
                g.correlation = genes[gene]
                gi = GeneInfo.objects.filter(gene_id=gene)
                if gi:
                    g.description = gi[0].description
                    g.gene_name = gi[0].gene_name
                    g.chr = gi[0].chr
                    g.start = gi[0].start
                genelist.append(g)

            return render_to_response('correlation.html', {'experiment_name': exp_name,
                                                           'experiments': experiments,
                                                           'species': species_short_name,
                                                           'queryGene': queryGene,
                                                           'gene_list': sorted(genelist, key=lambda x: x.correlation,
                                                                               reverse=True),
                                                           'corrthld': corrthld})
        else:
            return render_to_response('correlation.html', {'experiments': experiments})


def getCorrelatingGenes(correlations, corrthld):
    genes = dict()
    for gene in correlations:
        if abs(correlations[gene]) >= corrthld:
            genes[gene] = correlations[gene]

    return genes


def loadLodFile(experiment):
    lod_path = Experiment.objects.get(experiment_name=experiment).lod_file
    lod_file_path = os.path.join(settings.MEDIA_ROOT, lod_path)

    lodscores = dict()
    with open(lod_file_path) as lod_file:
        header = lod_file.readline()

        for line in lod_file.readlines():
            fields = line.strip().split('\t')
            geneID = fields[0].upper()
            lodscores[fields[0]] = fields[1:]

    return lodscores


def calculateCorrelation(gene, lodscores):
    geneLodscores = [abs(float(x)) for x in lodscores[gene]]
    correlations = dict()
    for s in lodscores:
        correlations[s] = numpy.corrcoef(geneLodscores, [abs(float(x)) for x in lodscores[s]])[0, 1]
    return correlations


def loadPearsonFile(gene, experiment):
    lod_path = Experiment.objects.get(experiment_name=experiment).lod_file
    pearson_path = re.sub(r'/[^/]*$', "/pearson.csv", lod_path)
    index_path = re.sub(r'/[^/]*$', "/pearson.json", lod_path)
    pearson_file_path = os.path.join(settings.MEDIA_ROOT, pearson_path)
    index_json_path = os.path.join(settings.MEDIA_ROOT, index_path)

    correlations = dict()

    geneIndex = None
    with open(index_json_path) as index_file:
        geneIndex = json.load(index_file)

    if gene not in geneIndex:
        return correlations

    with open(pearson_file_path) as pearson_file:
        header = pearson_file.readline().strip();
        pearson_file.seek(geneIndex[gene], 0)
        line = pearson_file.readline().strip();
        genes = header.split(",")
        fields = line.split(",")
        for g in range(1, len(genes)):
            try:
                correlations[genes[g]] = float(fields[g])
            except ValueError:
                print fields[g], g, genes[g]

    return correlations


def createPearsonFileIndex(experiment):
    lod_path = Experiment.objects.get(experiment_name=experiment).lod_file
    pearson_path = re.sub(r'/[^/]*$', "/pearson.csv", lod_path)
    index_path = re.sub(r'/[^/]*$', "/pearson.json", lod_path)
    pearson_file_path = os.path.join(settings.MEDIA_ROOT, pearson_path)
    index_json_path = os.path.join(settings.MEDIA_ROOT, index_path)

    geneIndex = dict()
    with open(pearson_file_path) as pearson_file:
        line = pearson_file.readline()
        while line:
            filePos = pearson_file.tell()
            line = pearson_file.readline()
            geneID = line.split(',', 1)[0]
            if geneID:
                geneIndex[geneID.upper()] = filePos

    with open(index_json_path, 'w') as index_file:
        json.dump(geneIndex, index_file)
