import json
import os

import urllib
import sys

from main.models import Experiment, GeneGO,ArraySpot,Transcript,Marker, Species, Chromosome, GeneInfo
from math import factorial
from django.conf import settings


def testGOaccession(exp_name,genelist,GO_accession):
    GI = GeneInfo.objects.filter(arrayspot__experiment__experiment_name = exp_name)
    ctotalGenes = GI.count()

    GIGO = GI.filter(genego__term_accession = GO_accession)
    ctotalGenesAccession = GIGO.count()
    cgenelistAccession = GIGO.filter(gene_id__in = genelist).count()
    cgenelist = len(genelist)
    N = ctotalGenes 
    m1 = 1
    m2 = N - m1
    n1 = 1
    n2 = 1
    a = 1
    b = 1
    c = 1
    d = 1
    pvalue = factorial(m1)*factorial(m2)*factorial(n1)*factorial(n2)/(factorial(N)*factorial(a)*factorial(b)*(factorial(c)*(factorial(d))))
    return pvalue

def loadGO_OBO(filename): 
    with open(filename) as GO_OBO_File:
        for line in GO_OBO_File:
            pass

def getChrMarkers(experiment_name, chromosome):
    return Marker.objects.filter(experiment__experiment_name = experiment_name, chromosome__name = chromosome)

def getClosestMarker(experiment_name, chromosome, pos):
    pos = int(pos)
    markers = getChrMarkers(experiment_name, chromosome)
    minDist = -1
    closestMarker = None
    for marker in markers:
        dist = abs(pos - marker.start)
        if minDist == -1 or dist < minDist:
            minDist = dist
            closestMarker = marker

    return closestMarker

def getMarkerObjects(experiment_name):
    return Marker.objects.filter(experiment__experiment_name = experiment_name)

def getSpeciesForExperiment(experiment_name):
    return Species.objects.get(experiment__experiment_name = experiment_name).species_name

def getMarkerNames(experiment_name):
    return Marker.objects.filter(experiment__experiment_name = experiment_name).values_list("name", flat=True)

def createSettingsJsonForExperiment(experiment_name):
    
    output_dic = dict()

    species_name = getSpeciesForExperiment(experiment_name)
    output_dic['species'] = species_name

    url = Species.objects.filter(species_name=species_name).values_list('url', flat=True)
    if len(url) == 1:
	output_dic['url'] = url[0]
    else:
        output_dic['url'] = ""

    # only report chromosomes for with markers are
    chromosomes = Chromosome.objects.filter(marker__experiment__experiment_name=experiment_name).distinct()
    output_dic['chrnames'] = sorted(chromosomes.values_list('name', flat=True))

    chr_dic = dict()
    for chromosome in chromosomes:
        chr_dic[chromosome.name] = {"start": chromosome.start, "end": chromosome.end}
	
    output_dic["chr"] = chr_dic

    out_file = os.path.join(settings.MEDIA_ROOT,'data/%s/%s.json' % (experiment_name,"settings"))

    with open(out_file, 'w') as fo:
        json.dump(output_dic, fo, indent=4, ensure_ascii=True)

def createGeneJsonForExperiment(experiment_name,qtl_file):
    '''
    description: parse upload information into JSON file format later which later will be used for multiexperimentplot plot. 
    
    @type experiment_name: name of experiment
    @param experiment_name: string
    
    @type qtl_file: qtl mapping output
    @param qtl_file: file
    
    @type thld: LOD theshold
    @param thld: decimal

    @rtype output: path of output file
    @return output: string
    
    '''
    
    in_path = os.path.join(settings.MEDIA_ROOT,'data/%s/%s' % (experiment_name,qtl_file))

    species_name = getSpeciesForExperiment(experiment_name)
    geneInfo_list = GeneInfo.objects.filter(species__species_name=species_name)
    geneInfo_dict = dict((geneInfo.gene_id, geneInfo) for geneInfo in geneInfo_list)

    marker_list = Marker.objects.filter(experiment__experiment_name=experiment_name)
    marker_dict = dict((marker.name, marker) for marker in marker_list)

    with open(in_path) as fi:
        first_line = fi.readline().rstrip()
        markers = first_line.split('\t')[1:]

        for line in fi:
            output_dic = {}
            fields = line.split('\t')
            gene = fields[0].upper()

            output_dic["gene"] = gene

            if gene not in geneInfo_dict:
                sys.stderr.write("gene_id not in database: %s, (species_name: %s)\n"%(gene, species_name))
                continue
            geneInfo = geneInfo_dict[gene]
            output_dic["name"] = geneInfo.gene_name
            output_dic["chr"] = geneInfo.chr
            output_dic["start"] = geneInfo.start
            output_dic["end"] = geneInfo.end
            output_dic["experiment"] = experiment_name
            lodscore_dict = {}
            for idx, markerName in enumerate(markers):
                if markerName not in marker_dict:
                    sys.stderr.write("marker not in database: %s\n" % markerName)
                    continue
                marker = marker_dict[markerName]
                lodscore_dict[markerName] = dict(chr=marker.chromosome.name, start=marker.start, end=marker.end,
                                               lod=fields[idx + 1])

            output_dic["lodscores"] = lodscore_dict

            out_file = os.path.join(settings.MEDIA_ROOT, 'data/%s/genes/%s.json' % (experiment_name, gene))

            with open(out_file, 'w') as fo:
                json.dump(output_dic, fo, indent=4)

            probe_output_dic = dict()
            probe_output_dic["probe"] = gene
            probe_output_dic["lod"] = list(map(float, fields[1:]))
            probe_out_file = os.path.join(settings.MEDIA_ROOT, 'data/%s/probe/%s.json' % (experiment_name, gene))

            with open(probe_out_file, 'w') as fo:
                json.dump(probe_output_dic, fo, indent=4)
